<?php

/**
 * Ajax handler for updating form for creating fields.
 */
function quick_data_update_field_form($form, $form_state) {
  return $form['settings'];
}


/**
 * Quick data form routeur.
 */
function quick_data_add_field_form($form, &$form_state){

    if (!isset($form_state['stage'])) $form_state['stage'] = 'quick_data_collect_field_label';

    switch ($form_state['stage']) {
        case 'quick_data_collect_field_label':
            return quick_data_collect_field_label($form, $form_state);
            break;

        case 'quick_data_create_field_data':
            return quick_data_create_field_data($form, $form_state);
            break;

        default:
            return quick_data_collect_field_label($form, $form_state);
            break;
    }
}
/*
 * Quick data all step submit
 */
function quick_data_add_field_form_submit($form, &$form_state){
    $values = isset($form_state['multistep_values']['quick_data_collect_field_label']) ? $form_state['multistep_values']['quick_data_collect_field_label'] : array();
    switch ($form_state['stage']) {
        case 'quick_data_create_field_data':
            $form_state['multistep_values'][$form_state['stage']] = $form_state['values'];
            if ($form_state['triggering_element']['#value'] != 'Back') {
                quick_data_create_field_data_submit($form, $form_state);
                $form_state['complete'] = TRUE;
            }
            break;

        default:
            $form_state['multistep_values'][$form_state['stage']] = $form_state['values'];
            $form_state['new_stage'] = quick_data_add_field_move_to_next_stage($form, $form_state);
            break;
    }

    if (isset($form_state['complete'])){
        if(empty($values['bundle'])){
            $values['bundle'] = strtolower(str_replace(' ','_',$values['name']));
        }
        switch ($values['type']) {
            case 'node':
                $goto = 'admin/structure/types/manage/'.$values['bundle'].'/fields';
                break;
            case 'user':
                $goto = 'admin/config/people/accounts/fields';
                break;
            case 'taxonomy_term':
                $goto = 'admin/structure/taxonomy/'.$values['bundle'].'/fields';
                break;
            case 'field_collection_item':
                $goto = 'admin/structure/field-collections/'.$values['bundle'];
                break;
            case 'paragraphs_item':
                $goto = 'admin/structure/paragraphs/'.$values['bundle'].'/fields';
                break;
            case 'data':
                $goto = 'admin/structure/data/edit/'.$values['bundle'];
                break;
            default:
                $goto = 'admin/structure/types/manage/article/fields';
        }
        drupal_goto($goto);
    }

    if ($form_state['triggering_element']['#value'] == 'Back') {
        $form_state['new_stage'] = quick_data_add_field_move_to_previous_stage($form, $form_state);
    }

    if (isset($form_state['multistep_values']['form_build_id'])) {
        $form_state['values']['form_build_id'] = $form_state['multistep_values']['form_build_id'];
    }
    $form_state['multistep_values']['form_build_id'] = $form_state['values']['form_build_id'];
    $form_state['stage'] = $form_state['new_stage'];
    $form_state['rebuild'] = TRUE;
}

/**
 * Given the current stage the user is on, calculate what the next step would be
 *
 * @param type $form_state
 * @return string
 */
function quick_data_add_field_move_to_next_stage($form, &$form_state) {
    switch ($form_state['stage']) {
        case 'quick_data_collect_field_label':
            return 'quick_data_create_field_data';
            break;
        case 'quick_data_create_field_data':
            return 'quick_data_collect_field_label';
            break;
    }
}
/**
 * Given the current stage the user is on, calculate what the previous step
 * would be
 *
 * @param type $form_state
 * @return string
 */
function quick_data_add_field_move_to_previous_stage($form, &$form_state) {
    switch ($form_state['stage']) {
        case 'quick_data_create_field_data':
            return 'quick_data_collect_field_label';
            break;
    }

}
/*
 * Quick data form 1st step
 */
function quick_data_collect_field_label($form, &$form_state) {

    $values = isset($form_state['multistep_values']['quick_data_collect_field_label']) ? $form_state['multistep_values']['quick_data_collect_field_label'] : array();
    $option = array(
        'user' => t('User'),
        'node' => t('Node'),
        'taxonomy_term' => t('Taxonomy'),
    );
    if( module_exists('field_collection')){
        $option['field_collection_item'] = t('Field collection');
    }
    if( module_exists('paragraphs')){
        $option['paragraphs_item'] = t('Paragraphs');
    }
    if( module_exists('data')){
        $option['data'] = t('Data');
    }
    $form['collect_field_label']['type'] = array(
        '#type' => 'select',
        '#title' => t('Type'),
        '#options' => $option,
        '#default_value' => isset($values['type']) ? $values['type'] : 'user',
        '#description' => t('Set your Type'),
        '#ajax' => array(
            'callback' => 'ajax_quick_data_collect_field_callback',
            'wrapper' => 'quick-data-add-field-type',
            'method' => 'replace',
        ),
    );
    $type = !empty($form_state['values']['type'])? $form_state['values']['type']:null;
    if(empty($type) && !empty($values['type'])){
        $type =  $values['type'];
    }
    $form['collect_field_label']['bundle'] = array(
        '#type' => 'select',
        '#title' => t('Bundle'),
        '#empty_option' => t('- New bundle -'),
        '#options' =>  _collect_field_label_type($type),
        '#default_value' => isset($values['bundle']) ? $values['bundle'] : NULL,
        '#description' => t('Set your bundle'),
        '#states' => array(
            'invisible' => array(
                ':input[name="type"]' =>  array('value' => 'user'),
            ),
        ),
        '#prefix' => '<div id="quick-data-add-field-type">',
        '#suffix' => '</div>',
    );
    $form['collect_field_label']['name'] = array(
        '#type' => 'textfield',
        '#title' => t('Machine name'),
        '#maxlength' => 32,
        '#default_value' => isset($values['name']) ? $values['name'] : NULL,
        '#description' => t('Set your machine name, if new bundle'),
        '#states' => array(
            'invisible' => array(
                ':input[name="type"]' =>  array('value' => 'field_collection_item'),
            ),
            'visible' => array(
                ':input[name="bundle"]' => array('value' => ''),
            ),
        ),
    );
    $form['collect_field_label']['field_exist'] = array(
        '#type' => 'checkbox',
        '#title' => t('Exist field'),
        '#default_value' => isset($values['field_exist']) ? $values['field_exist'] : NULL,
        '#description' => t('If you want load exits field'),
        '#states' => array(
            'invisible' => array(
                ':input[name="bundle"]' => array('value' => ''),
            ),
        ),
        '#ajax' => array(
            'callback' => 'ajax_quick_data_collect_label_callback',
            'wrapper' => 'quick-data-add-field-label',
            'method' => 'replace',
        ),
    );

    $form['collect_field_label']['field_label'] = array(
        '#type' => 'textarea',
        '#title' => t('Field label'),
        '#description' => t('Enter your list field label'),
        '#default_value' => isset($values['field_label']) ? $values['field_label'] : NULL,
        '#prefix' => '<div id="quick-data-add-field-label">',
        '#suffix' => '</div>',
    );
    if( empty($form['collect_field_label']['field_label']['#default_value']) ){
      if(!empty($form_state["values"]["field_exist"])){
        $form['collect_field_label']['field_label']['#value'] = _load_type_bunlde_fields_info($form_state["values"]["type"],$form_state["values"]["bundle"]);
      }elseif(isset($form['collect_field_label']['field_label']['#value'])){
        unset($form['collect_field_label']['field_label']['#value']);
      }
    }
    $generate_module = [
        'search_api' => false,
        'feeds' => false,
        'views' => false
    ];
    $module_names = [];
    foreach ($generate_module as $module=>&$existe){
        if(module_exists($module)) {
            $existe = true;
            $info = system_get_info('module',$module);
            $module_names[$module] = $info['name'];
        }
    }
    $form['collect_field_label']['generate'] = array(
        '#type' => 'fieldset',
        '#title' => t('Extra generate'),
        '#collapsible' => TRUE, // Added
        '#collapsed' => FALSE,  // Added
        '#description' => t('Gererate with modules ').implode(', ',$module_names),
    );
    if(!empty($module_names)){

        foreach ($module_names as $module=>$module_name){
            $form['collect_field_label']['generate'][$module] = array(
                '#type' => 'checkbox',
                '#title' => $module_name,
                '#default_value' => isset($values['generate'][$module]) ? $values['generate'][$module] : 1,
            );
            if($module == 'search_api'){
                $servers_search_api  = search_api_server_load_multiple(false);
                $optionServerSearch = [];
                if(!empty($servers_search_api)){
                    foreach ($servers_search_api as $server_search_api => $search_api){
                        $optionServerSearch[$search_api->machine_name] = $search_api->name;
                    }
                }
                $form['collect_field_label']['generate']['SearchApiServer'] = array(
                    '#type' => 'select',
                    '#title' => t('Search Server'),
                    '#empty_option' => t('- New Search Server -'),
                    '#description' => t('Must have server like ').l('Database Search','https://www.drupal.org/project/search_api_db').t(' or ') . l('Elasticsearch','https://www.drupal.org/project/search_api_elasticsearch'),
                    '#options' =>  $optionServerSearch,
                    '#default_value' => isset($values['generate']['SearchApiServer']) ? $values['generate']['SearchApiServer'] : NULL,
                    '#states' => array(
                        'visible' => array(
                            "input[name='search_api']" => array("checked" => TRUE),
                        ),
                    ),
                );
            }
            // paragraphs_item don't support feeds
            if($module == 'feeds'){
                $form['collect_field_label']['generate'][$module]['#states'] = array(
                    'invisible' => array(
                        ':input[name="type"]' =>  array('value' => 'paragraphs_item'),
                    ),
                );
            }
        }
    }
    $form['next'] = array(
        '#type' => 'submit',
        '#value' => t('Next')
    );
    return $form;
}

// Ajax Call
function ajax_quick_data_collect_field_callback(&$form, &$form_state) {
    return $form['collect_field_label']['bundle'];
}
function ajax_quick_data_collect_label_callback(&$form, &$form_state) {
    return $form['collect_field_label']['field_label'];
}
function _load_type_bunlde_fields_info($type = null,$bundle = null){
  if(is_null($type) && is_null($bundle)){
    return null;
  }
  $params = [
    'entity_type' => $type,
  ];
  if(!empty($bundle)){
    $params ['bundle'] = $bundle;
  }
  $fields = field_read_instances($params);
  foreach ($fields as $field){
    $field_info = field_info_field($field['field_name']);
    $out [] = $field['field_name'].'|'.$field['label'].'|'.$field_info["type"].'|'.$field["widget"]["type"];
  }
  return implode("\n",$out);
}
/*
 * get all type of bundle
 */
function _collect_field_label_type($bundle=false){
    $type = [];
    if(!empty($bundle) && !in_array($bundle,['user','data'])){
        $entity_type = entity_get_info( $bundle);
        if(!empty($entity_type['bundles'])){
            foreach ($entity_type['bundles'] as $key=>$name){
                $type[$key]=$name['label'];
            }
        }
    }
    return $type;
}
/*
 * return machine name of bundle
 */
function _collect_field_machine_names_type($bundle){
    if(!empty($bundle) && $bundle!='user'){
        $entity_type = entity_get_info( $bundle);
        if(!empty($entity_type['bundles'])){
            return array_keys ($entity_type['bundles']);
        }
    }
    return [];
}
/*
 * Quick data form step 2
 */
function quick_data_create_field_data($form, &$form_state) {

    $form['#value'] = $form_state['values'];
    $values = isset($form_state['multistep_values']['quick_data_create_field_data']) ? $form_state['multistep_values']['quick_data_create_field_data'] : array();

    $form['back'] = array(
        '#type' => 'submit',
        '#value' => t('Back')
    );

    $form['next'] = array(
        '#type' => 'submit',
        '#value' => t('Submit')
    );

    return $form;
}
/*
 * Quick data form on submit
 */
function quick_data_create_field_data_submit($form, &$form_state) {
    $values = !empty($form_state['multistep_values']['quick_data_collect_field_label'])?$form_state['multistep_values']['quick_data_collect_field_label']:false;
    if(!empty($form_state['input']['fields'])){
        $bundle = $values['bundle'];
        $name = ucfirst(str_replace('_',' ',$bundle));
        $entity_type = $values['type'];
        $new_name = $values['name'];
        // Define the node type.
        switch ($entity_type){
          case 'node' :
            $feeds_processor = array(
              'plugin_key' => 'FeedsNodeProcessor',
              'config' => array(
                'expire' => '-1',
                'author' => 0,
                'authorize' => 0,
              ),
            );
            break;
          case 'taxonomy_term' :
            $vocabulary = taxonomy_vocabulary_machine_name_load($name) ;
            if(!$vocabulary){
              $vocabulary = new stdClass();
              $vocabulary->name = $name;
              $vocabulary->machine_name = $new_name;
              $vocabulary->description = t('Create a new taxonomy ').' '.$name;
              $vocabulary->module = 'taxonomy';
              taxonomy_vocabulary_save($vocabulary);
            }
            $feeds_processor = array(
              'plugin_key' => 'FeedsTermProcessor',
              'config' => array(
                'vocabulary' => $vocabulary->vid,
                'bundle' => !empty($new_name)? $new_name:$bundle,
              ),
            );
            break;
          case 'paragraphs_item' :
            $feeds_processor = false;// not support paragraphs
            break;
          case 'user' :
            $feeds_processor = array(
              'plugin_key' => 'FeedsUserProcessor',
              'config' => array(
                'bundle' => null,
                'roles' => array(),
                'status' => 1,
                'defuse_mail' => FALSE,
              ),
            );
            break;
          case 'field_collection_item' :
            $feeds_processor = array(
              'plugin_key' => 'FeedsFieldCollectionProcessor',
              'config' => array(
                'field_name' => $bundle,
                'host_entity_type' => $entity_type,
                'host_entity_importer_id' => '',
                'host_entity_importer_nid' => NULL,
                'guid_field_name' => '',
                'is_field' => 1,
              ),
            );
            break;
          case 'data' :
            break;
        }
        $get_instance = field_info_instances($entity_type, $bundle);

        if( !empty($entity_type) && !empty($new_name) && empty($get_instance[$new_name])){
            $bundle = strtolower(str_replace(' ','_',$new_name));
            $name = ucfirst($new_name);
            if($entity_type=='node'){
                $new_content_type = array(
                    'type'          => $bundle,
                    'name'          => $name,
                    'description'   => t('Create a new content ').' '.$name,
                    'title_label'   => t('Title'),
                    'base'          => 'node_content',
                    'custom'        => TRUE,
                );
                $node_type = node_type_set_defaults($new_content_type);
                // Save the content type
                node_type_save($node_type);
            }
            if($entity_type=='paragraphs_item'){
                $bundle_paragraphs = new stdClass();
                $bundle_paragraphs->bundle = $bundle;
                $bundle_paragraphs->locked  = true;
                $bundle_paragraphs->name = $name;
                paragraphs_bundle_save($bundle_paragraphs);
            }
            if($entity_type=='data'){
              foreach ($form_state['input']['fields'] as $field_info) {
                $field = [
                  'name' => $field_info['field_name'],
                  'type' => $field_info['type'],
                  'size' => $field_info['widget_type']
                ];
                $schema['fields'][$field_info['field_name']] = data_build_field_definition($field);
                $meta['fields'][$field_info['field_name']]['label'] = check_plain($field_info['label']);
              }
              if(!empty($schema)){
                $bundle_data = data_create_table($bundle, $schema, $new_name);
                $meta = $bundle_data->update(array('meta' => $meta));
                //@todo will remove if finish views && feeds
                drupal_goto('admin/structure/data/edit/'.$bundle);
              }
            }
        }

        if($entity_type == 'user'){
          $bundle = 'user';
	        $name = ucfirst($bundle);
        }

        $type_index = [
            'text' => 'text',
            'text_long' => 'text',
            'field_hidden_text_long' => 'text',
            'field_hidden_text' => 'text',
            'office_hours' => 'text',
            'integer' => 'integer',
            'field_hidden_integer' => 'integer',
            'number_integer' => 'integer',
            'list_integer' => 'integer',
            'entityreference' => 'integer',
            'decimal' => 'decimal',
            'field_hidden_decimal' => 'decimal',
            'list_float' => 'decimal',
            'number_decimal' => 'decimal',
            'number_float' => 'decimal',
            'field_hidden_float' => 'decimal',
            'date' => 'date',
            'datetime' => 'date',
            'datestamp' => 'date',
            'field_timetable_timetable' => 'date',
            'duration' => 'duration',
            'boolean' => 'boolean',
            'list_boolean' => 'boolean',
            'number_boolean' => 'boolean',
            'image' => 'uri',
            'file' => 'uri',
            'media' => 'uri',
            'uri' => 'uri',
        ];
        $type_feed_target = [
          'datetime' => ':start',
          'datestamp' => ':start',
          'date' => ':start',
          'file' => ':uri',
          'double_field' => ':first',
          'addressfield' => ':thoroughfare',
          'name' => ':family',
          'phone' => ':url'
        ];

        module_invoke_all('quick_data_feed_target', $type_feed_target);
        $fields_index['search_api_language'] = ['type'=>'string'];
        foreach ($form_state['input']['fields'] as $field_name_complet=>$field_info){
            $field_name = 'field_'.$field_info['field_name'];
            // Create the field base.
            if(!field_info_field($field_name)) {
                $field = array(
                    'field_name' => $field_name,
                    'type' => $field_info['type'],
                );
                field_create_field($field);
            }
            $target = !empty($type_feed_target[$field_info['type']])?$type_feed_target[$field_info['type']]:'';
            $feeds_mappings[] =  array(
              'source' => $field_info['label'],
              'target' => $field_name.$target,
              'unique' => FALSE,
            );
            $string_type_index = !empty($type_index[$field_info['type']])?$type_index[$field_info['type']]:'string';
            $fields_index[$field_name] = ['type'=>$string_type_index];
            $fields_view_index[$field_name] = ['id'=>$field_name,'field'=>$field_name,'table'=>'search_api_index_'.$bundle];
            $fields_view[$field_name] = ['id'=>$field_name,'field'=>$field_name,'table'=>'field_data_'.$field_name];

            // Create the field instance on the bundle.
            $instance = array(
                'field_name' => $field_name,
                'entity_type' => $entity_type,
                'label' => $field_info['label'],
                'bundle' => $bundle,
                'widget' => array(
                    'type' => $field_info['widget_type'],
                ),
            );
            $test_instance_exist = field_info_instance($entity_type, $field_name, $bundle) ;
            if(!$test_instance_exist){
                field_create_instance($instance);
            }
        }
    }

    if(!empty($values['search_api'])){
        if(empty($values['SearchApiServer'])){
          $search_api_class = FALSE;
          if(module_exists('search_api_elasticsearch_elastica')){
            $search_api_class = 'search_api_elasticsearch_elastica_service';
          }
          if(module_exists('search_api_db')){
            $search_api_class = 'search_api_db_service';
          }
          if($search_api_class) {
            $values_server = [
              'name' => 'Default db server',
              'machine_name' => 'default_db_server',
              'enable' => TRUE,
              'class' => $search_api_class
            ];
            $server = entity_create('search_api_server', $values_server);
            drupal_set_message(t('Create new search server ') . l($values_server['name'], 'admin/config/search/search_api/server/' . $values_server['machine_name'] . '/edit'));
          }
        }else{
            $server = entity_load('search_api_server',[$values['SearchApiServer']]);
            $server = end($server);
        }
        if(!empty($server)){
          $options = ['database' => 'default:default','min_chars'=>3,'partial_matches'=>1];
          $server->configurationFormSubmit([], $options, $form_state);
          $server->save();
          $bundles = ['bundles'=>array($bundle)];
          $options = ['index_directly'=>1,'cron_limit'=>50];
          $values_bundles = array(
            'name' => $name,
            'machine_name' => $bundle,
            'item_type' => $entity_type,
            'options' => $options,
            'server' => $server->machine_name,
            'read_only' => false,
            'enable' => true,
            'old_type' => $entity_type,
            'datasource_config' => false
          );
          $index = search_api_index_load($bundle);
          if(!$index){
            $search_api_index = search_api_index_insert($values_bundles);
            $index = search_api_index_load($search_api_index);
          }
          $options['datasource'] = $bundles;
          $options['fields'] = $fields_index;
          $index->update(['options' => $options]);
          drupal_set_message(t('Create new search index  ').l($name, 'admin/config/search/search_api/index/'.$bundle.'/edit'));
        }else{
          drupal_set_message(t('Cannot create new search server, please install ') . l('Search API Database Search', 'https://www.drupal.org/project/search_api_db'));
        }
    }
    if(!empty($values['views'])){
        $base_table = $bundle;
        if(!empty($values['search_api'])){
          $base_table = 'search_api_index_'.$bundle;
        }
        $view = views_get_view($bundle);
        if(empty($view)){
          $view = views_new_view();
          $view->name = $bundle;
        }else{
          $view->name = $bundle.rand(0, 99);
        }
        $view->tag = 'default';
        $view->base_table = $entity_type;
        if(!empty($values['search_api'])){
          $view->base_table = $base_table;
        }
        if($entity_type == "taxonomy_term") {
          $view->base_table = 'taxonomy_term_data';
        }
        if($entity_type == "user") {
          $view->base_table = 'users';
        }
        $view->human_name = ucfirst($name);
        $view->core = 7;
        $view->api_version = '3.0';
        $view->disabled = FALSE; /* Edit this to true to make a default view disabled initially */
        /* Display: Master */
        $handler = $view->new_display('default', 'Master', 'default');
        $handler->display->display_options= [
            'title' => ucfirst($name),
            'use_more_always' => FALSE,
            'access' => ['type' => 'none'],
            'cache' => ['type' => 'none'],
            'query' => ['type' => 'views_query'],
            'exposed_form' => ['type' => 'basic'],
            'pager'=> ['type' => 'full','options' => ['items_per_page' => 20]],
            'style_plugin' => 'table'
        ];

        if($entity_type == "user") {
          $handler->display->display_options['access']['type'] = 'perm';
          $handler->display->display_options['access']['perm'] = 'access user profiles';
        }

        if(!empty($values['search_api'])){
          $handler->display->display_options['fields'] = $fields_view_index;
        }else{
          switch ($entity_type) {
            case "node":
              if(!empty($fields_view)) {
                $handler->display->display_options['fields'] = $fields_view;
              }
              /* Champ: Content: Title */
              $handler->display->display_options['fields']['title'] = [
                'id' => 'title',
                'table' => 'node',
                'field' => 'title',
              ];
              /* Sort criterion: Content: Post date */
              $handler->display->display_options['sorts']['created'] = [
                'id' => 'created',
                'table' => 'node',
                'field' => 'created',
                'order' => 'DESC'
              ];
              /* Filter criterion: Content: Public */
              $handler->display->display_options['filters']['status'] = [
                'id' => 'status',
                'table' => 'node',
                'field' => 'status',
                'value' => 1,
                'group' => 1,
                'operator' => FALSE
              ];
              /* Filter criterion: Content: Type */
              $handler->display->display_options['filters']['type'] = [
                'id' => 'type',
                'table' => 'node',
                'field' => 'type',
                'value' => [$bundle => $bundle]
              ];
              break;
            case "user":

              if(!empty($fields_view)) {
                $handler->display->display_options['fields'] = $fields_view;
              }
              /* Champ: User: name */
              $handler->display->display_options['fields']['name'] = [
                'id' => 'name',
                'table' => 'users',
                'field' => 'name',
              ];

              /* Sort criterion: Utilisateur: Nom */
              $handler->display->display_options['sorts']['name'] = [
                'id' => 'name',
                'table' => 'users',
                'field' => 'name',
              ];

              /* Filter criterion: User: Active */
              $handler->display->display_options['filters']['status'] = [
                'id' => 'status',
                'table' => 'users',
                'field' => 'status',
                'value' => '1'
              ];
              break;
            case "taxonomy_term":

              if(!empty($fields_view)) {
                $handler->display->display_options['fields'] = $fields_view;
              }
              /* Champ: Term taxonomy: Name */
              $handler->display->display_options['fields']['name'] = [
                'id' => 'name',
                'table' => 'taxonomy_term_data',
                'field' => 'name',
                'link_to_taxonomy' => TRUE
              ];
              $vid = taxonomy_vocabulary_machine_name_load($bundle) ;
              /* Filter criterion: Term taxonomy: Vocabulary */
              $handler->display->display_options['filters']['vid'] = [
                'id' => 'vid',
                'table' => 'taxonomy_term_data',
                'field' => 'vid',
                'value' => array($vid => $vid)
              ];

              break;
            case "field_collection_item":

              /* Relationship: Field collection item: Entity with the field */
              //check entity reference
              $set_entity_type = 'node';
              foreach (['user','taxonomy_term'] as $get_entity_type){
                $entities = field_info_instances($get_entity_type);
                $flag = false;
                if(!empty($entities)){
                  foreach ($entities as $entity_check){
                    if(!empty($entity_check[$bundle])){
                      $set_entity_type = $entity_check[$bundle]["entity_type"] ;
                      break;
                    }
                  }
                }
                if($flag){
                  break;
                }
              }
              $handler->display->display_options['relationships'] = [
                $bundle.'_'.$set_entity_type => [
                  'id' => $bundle.'_'.$set_entity_type,
                  'table' => 'field_collection_item',
                  'field' => $bundle.'_'.$set_entity_type,
                  'required' => TRUE
                ]
              ];

              /* Champ: Field collection item: Field collection item ID */
              $handler->display->display_options['fields']['name'] = [
                'id' => 'item_id',
                'table' => 'field_collection_item',
                'field' => 'item_id',
                'separator' => ''
              ];
              foreach ($fields_view as $field_name_view=>$field_value_view){
                $handler->display->display_options['fields'][$field_name_view] = [
                  'id' => $field_name_view,
                  'table' => 'field_data_'.$field_value_view['field'],
                  'field' => $field_value_view['field']
                ];
              }
              break;
            case 'paragraphs_item':
              break;
            case 'data' :
              //@todo for data tables
              break;
          }
        }
        /* Display: Page */
        $handler = $view->new_display('page', 'Page', 'page');
        $path = str_replace('_','-',$bundle);
        if (drupal_valid_path($path) || drupal_lookup_path('source', $path)) {
          $path .= random_int(1,10);
        }
        $handler->display->display_options['path'] = $path;

        $view->execute_display('default', array());
        $view->save();
        drupal_set_message(t('Create new view ').l($name, 'admin/structure/views/view/'.$view->name.'/edit'));
    }
    if(!empty($values['feeds'])){
        $importer = feeds_importer($bundle);
        $config = array(
            'name' => $name,
            'id' => $bundle,
            'description' => t('Import CSV ').$name,
            'fetcher' => array(
                'plugin_key' => 'FeedsFileFetcher',
                'config' => array(
                    'allowed_extensions' => 'txt csv tsv xml opml',
                    'delete_uploaded_file' => FALSE,
                    'direct' => FALSE,
                    'directory' => 'public://feeds',
                    'allowed_schemes' => ['public'],
                ),
            ),
            'parser' => array(
                'plugin_key' => 'FeedsCSVParser',
                'config' => array(
                    'delimiter' => ',',
                    'encoding' => 'UTF-8',
                    'no_headers' => 0,
                ),
            ),
            'processor' => array(
                'config' => array(
                    'mappings' => $feeds_mappings,
                    'insert_new' => 1,
                    'update_existing' => 0,
                    'update_non_existent' => 'skip',
                    'input_format' => 'plain_text',
                    'skip_hash_check' => 0,
                    'bundle' => $bundle,
                    'language' => LANGUAGE_NONE,
                ),
            ),
            'content_type' => '',
            'update' => 0,
            'import_period' => -1,
            'expire_period' => 3600,
            'import_on_create' => 1,
            'process_in_background' => 0,
        );
        if(!empty($feeds_processor)){
            $config['processor'] =  $config['processor']+ $feeds_processor;
            $config['processor']['config'] =  $config['processor']['config']+ $feeds_processor['config'];

	        $importer->setConfig($config);
	        foreach (array('fetcher', 'parser', 'processor') as $type) {
		        $importer->setPlugin($config[$type]['plugin_key']);
		        $importer->$type->setConfig($config[$type]['config']);
	        }
            $importer->save();
            drupal_set_message(t('Create new feeds ').l($name, 'admin/structure/feeds/'.$bundle.'/mapping'));
        }
    }
    return $form;
}

/**
 * Defines configuration form fields.
 *
 * Implements drupal_get_form().
 *
 * @param array $form
 * @param array &$form_state
 * @return array
 *   - the return value of system_settings_form()
 */
function _quick_data_admin_form($form, &$form_state) {
  $form['quick_data_settings_stop_word'] = array(
    '#type' => 'textfield',
    '#title' => t('Stop word'),
    '#decription' => t('These words will will remove when quick data create machine name for each field.'),
    '#default_value' => variable_get('quick_data_settings_stop_word', 'le la du de des a au en'),
    '#required' => FALSE,
  );
  return system_settings_form($form);
}
/*
 * helper convert name to machine_name
 * machine name must be less than 26 characters
 */
function _convert_machine_name($field_label, $maxlen = 26){
  $field_label = str_replace(["field_","'","_"], " ", trim($field_label));
  $unicode = array(
    'a' => 'á|à|ả|ã|ạ|ă|ắ|ặ|ằ|ẳ|ẵ|â|ấ|ầ|ẩ|ẫ|ậ|å|æ|Á|À|Ả|Ã|Ạ|Ă|Ắ|Ặ|Ằ|Ẳ|Ẵ|Â|Ấ|Ầ|Ẩ|Ẫ|Ậ|Å|Æ|@|Д|д|Ą|ą',
    'd' => 'đ|Đ|Ḍ|ḍ|ð|Ď|ď',
    'e' => 'é|è|ẻ|ẽ|ẹ|ê|ế|ề|ể|ễ|ệ|É|È|Ẻ|Ẽ|Ẹ|Ê|Ế|Ề|Ể|Ễ|Ệ|Э|э|Ɛ|ɛ|Σ|ε|€|Ę|ę|Ě|ě',
    'i' => 'í|ì|ỉ|ĩ|ị|ï|Í|Ì|Ỉ|Ĩ|Ị|Ï',
    'o' => 'ó|ò|ỏ|õ|ọ|ô|ố|ồ|ổ|ỗ|ộ|ơ|ớ|ờ|ở|ỡ|ợ|ö|ø|Ó|Ò|Ỏ|Õ|Ọ|Ô|Ố|Ồ|Ổ|Ỗ|Ộ|Ơ|Ớ|Ờ|Ở|Ỡ|Ợ|°|¤|Œ|œ|Ø|Ф|ф|Ю|ю|Ѳ|ѳ',
    'u' => 'ú|ù|ủ|ũ|ụ|ư|ứ|ừ|ử|ữ|ự|û|ü|Ú|Ù|Ủ|Ũ|Ụ|Ư|Ứ|Ừ|Ử|Ữ|Ự|Ü|Û|Ц|ц',
    'y' => 'ý|ỳ|ỷ|ỹ|ỵ|ÿ|Ý|Ỳ|Ỷ|Ỹ|Ỵ|Ÿ|¥|Ч|ч|Ɣ|ɣ|γ',
    '2' => '²',
    'c' => 'ç|Ç|Ć|ć',
    's' => 'š|Š|§|Ṣ|ṣ|Ś|ś|Ş|ş',
    'l' => '£|Ł|ł|Ĺ|ĺ|Ľ|ľ',
    'm' => 'μ',
    'n' => 'И|и|ñ|Ñ|Л|л|П|п|Ń|ń',
    'z' => 'Ž|ž|Ẓ|ẓ|Ź|ź|Ż|ż',
    'b' => 'ß|ẞ|Ъ|ъ|Ы|ы|Б|б|Ѣ|ѣ|þ|Þ',
    'x' => 'Ж|ж',
    'w' => 'Ш|ш|Щ|щ|ŵ|Ŵ',
    'r' => 'Я|я|Γ|Ṛ|ṛ',
    'v' => 'Ѵ|ѵ',
    'g' => 'Ǧ|ǧ',
    'h' => 'Ḥ|ḥ',
    't' => 'Ṭ|ṭ|Ť|ť'
  );
  foreach ($unicode as $nonUnicode => $uni) {
    $field_label = trim(preg_replace("/($uni)/i", $nonUnicode, $field_label));
  }
  setlocale(LC_CTYPE, 'en_AU.utf8');
  $machine_name = preg_replace('/[^a-z0-9 ]/i', '', iconv("UTF-8", "US-ASCII//TRANSLIT", $field_label));

  $stop_word = variable_get('quick_data_settings_stop_word');
  if(!empty($stop_word)) {
    $words = explode(' ',strtolower($machine_name));
    if($words[0]=='field'){
      unset($words[0]);
    }
    $remove_word = explode(' ',strtolower($stop_word));
    $field_label = array_diff($words, $remove_word);
    $machine_name = implode(' ', $field_label);
  }

  $machine_name = substr($machine_name, 0, $maxlen);
  return $machine_name = str_replace('-', '_', drupal_html_id($machine_name));
}
